/*
 * Mach Operating System
 * Copyright (c) 1991,1990,1989 Carnegie Mellon University
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon
 * the rights to redistribute these changes.
 */
/*
 * Amiga-specific initialization.
 *
 * Contributors:
 *      Bryan Ford
 *      Niklas Hallqvist
 */

#include <mach/std_types.h>
#include <mach/machine.h>
#include <machine/custom.h>
#include <machine/cia.h>
#include <sys/types.h>
#include <sys/time.h>
#ifdef A3000
#include <machine/a3000.h>
#endif
#ifdef A2091
#include <machine/a2091.h>
#endif
#ifdef GVP11
#include <machine/gvp11.h>
#endif

char version[] = "0.009";

char *root_name = "blah";

int boothowto = 0;

vm_size_t mem_size;

extern char start[];
char *etext_ptr;

vm_offset_t virt_start, virt_end;

void *chipmem_start, *chipmem_end;


/* Called by the console et al to steal chip memory during initialization */
void *chipmem_steal(long amount)
{
  void *ptr = chipmem_start;
  chipmem_start += amount;
  if(chipmem_start > chipmem_end)
    panic("not enough chip memory");
  return(ptr);
}

/* Called from start.s, directly after being entered.  We are running in temporary virtual memory.  */
volatile void
start_c(int phys_stext,int phys_etext,int phys_size,int chipmem_size)
{
  int phys_end = phys_stext + phys_size;

  etext_ptr = start + phys_etext - phys_stext;
  mem_size = phys_size;

/*
  first_addr = move_bootstrap();
  first_addr = round_page(first_addr);
*/

  /* Set up our permanent kernel page tables and switch to them */
  pmap_bootstrap(phys_stext,&phys_etext,&phys_end,&virt_start,&virt_end);

  /* Map the standard hardware into its permanent addresses */
  pmap_map(VM_CUSTOM, PHYS_CUSTOM, PHYS_CUSTOM+PAGE_SIZE, VM_PROT_READ|VM_PROT_WRITE);
  pmap_map(VM_CIAA, PHYS_CIAA, PHYS_CIAA+PAGE_SIZE, VM_PROT_READ|VM_PROT_WRITE);
  pmap_map(VM_CIAB, PHYS_CIAB, PHYS_CIAB+PAGE_SIZE, VM_PROT_READ|VM_PROT_WRITE);

#ifdef A3000
  pmap_map(VM_A3000_DMAC, PHYS_A3000_DMAC, PHYS_A3000_DMAC+PAGE_SIZE, VM_PROT_READ|VM_PROT_WRITE);
#endif
#ifdef A2091
  pmap_map(VM_A2091_DMAC, PHYS_A2091_DMAC, PHYS_A2091_DMAC+PAGE_SIZE, VM_PROT_READ|VM_PROT_WRITE);
#endif
#ifdef GVP11
  pmap_map(VM_GVP11_DMAC, PHYS_GVP11_DMAC, PHYS_GVP11_DMAC+PAGE_SIZE, VM_PROT_READ|VM_PROT_WRITE);
#endif



  /* Get our chip memory allocation system working */
  chipmem_start = (void*)VM_MIN_CHIP_ADDRESS;
  chipmem_end = (void*)VM_MIN_CHIP_ADDRESS + chipmem_size;
  pmap_map(VM_MIN_CHIP_ADDRESS, 0, chipmem_size, VM_PROT_READ|VM_PROT_WRITE);


  /* Set up the trap system */
  /* XXX - Things will be faster if we move this out of chip memory, but currently SIM needs it there.  */
  load_vbr(chipmem_steal(sizeof(long)*256));
  trap_init();

  cninit();

  /* XXX */ startrtclock();

  printf("KludgeMach %s\n", version);
  printf("init calling setup_main\n");
  setup_main();
  /*NOTREACHED*/
}


volatile void
halt_all_cpus(void)
{
  asm volatile("movew #0x2700,sr;"
	       "movel #0x003f810f,sp@-;" /*pmove sp@,tt0;*/
	       "illegal;"
	       "jmp 2");
  /*NOTREACHED*/
}


volatile void
halt_cpu(void)
{
  int x = time.seconds;
  while (x != time.seconds + 2)
    ;
  asm volatile("illegal");
  asm volatile("stop #0x2700");
  /*NOTREACHED*/
}


static volatile void
halt (char *msg)
{
  asm ("illegal");
  while (1)
    ;
}


void
gimmeabreak(void)
{
  asm("illegal");
}

int timemmap(int dev_number,vm_offset_t ofs,vm_prot_t prot)
{
  return(0);
}


void delay(int per)
{
  /* Hmmm... */
#ifdef M68040
  /* The preprocessor condition above are really testing for a 68040 MMU
     but as there are no test for the 040 CPU I'm aware of, I'm using it.
     This delay has been shown to be needed on 040s >= 33 MHz, but not on
     28 MHz machines. */
  int i;
  for (i = per * 8; i; --i) ;
#endif
}


#include <mach/mach_interface.h>
#include <mach/thread_status.h>

/*
int
ex_get_header(struct file *fp,struct loader_info *lp)
{
  int result;
  vm_size_t resid;

  result = read_file(fp,0,lp,sizeof(*lp),&resid);
  if (result)
    return (result);
  if (resid)
    return (EX_NOT_EXECUTABLE);
  return(0);
}

char *
set_regs(mach_port_t user_task, mach_port_t user_thread, struct loader_info *lp, int arg_size)
{
  struct thread_state_regs regs;

  asm("illegal");
  regs.sp = VM_MAX_ADDRESS-arg_size-4;
  regs.pc = lp->text_start;
  thread_set_state(user_thread,THREAD_STATE_REGS,&regs,sizeof(regs)/sizeof(int));
  return(VM_MAX_ADDRESS-arg_size);
}
*/
